# Copyright 2004 Laurent GUEGUEN
# 
# This file is part of Sarment.
# 
# Sarment is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Sarment is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Sarment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import math
import string

import lcompte
from Modules.Clexique import *

import matrice
import sequence

class Lexique:
    "Lexique"

    def __init__(self, **kw):
        """ Keyword argument:
fic -- Build from filename fic;
fprop -- Build from Lproportion file of name fprop;
str -- Build from string str;
alpha -- alph (with keyword str), uses alphabet alpha.
"""
        self.__lex=Clexique()
        if kw.has_key('fic'):
            self.read_nf(kw['fic'])
        if kw.has_key('fprop'):
            lp=lcompte.Lproportion(fic=kw['fprop'])
            self.read_Lprop(lp)
            del lp
        if kw.has_key('str'):
            self.read_str(kw['str'],kw.get('alpha',""))

##################################
##  creation
        
    def read_nf(self,nf,ns=""):
        """Build from filename nf and alphabet ns=""."""
        self.__lex.read_fich(nf,ns)

    def read_str(self,nf,ns=""):
        """Build from string nf and alphabet ns=""."""
        self.__lex.read_str(nf,ns)

    def __setitem__(self, i, st):
        """Add a descriptor to a specific number, or a descriptor pattern
to a specific tuple.
"""
        lsn=self.ls_num()
        if type(i)==int:
            if i in lsn:
                print "Bad descriptor number ", i, " already used"
                return
            s=str(i)
        elif type(i)==tuple:
            s=""
            for a in i:
                if a in lsn:
                    print "Bad descriptor number ", a, " already used"
                    return
                s+=str(a)+','
            s=s[:-1]
        else:
            print "Bad descriptor type ", i
            return
        
        self.__lex.add_str(s+":"+st)
        
    def read_Lprop(self, lp):
        """Build from Lproportion lp. Compute automatically
log-probabilites, or -10000 if log not possible.
"""
        lpn=lp.num()
        ret=""
        for i in lpn:
            ret+=str(i)+":"+lp[i].loglex()+" "

        self.read_str(ret)
        self.__lex.init_trans();
        for d in lpn:
            for f in lpn:
                x=lp.inter(d,f)
                if x>0:
                    x=math.log(x)
                else:
                    x=-10000
                if self.__lex.g_trans(d,f,x)==0:
                    print "Problem of transition between %d and %d"%(d,f)

    def read_prop(self, p):
        """Build from Proportion p. Compute automatically
log-probabilites, or -10000 if log not possible.
"""
        self.read_str(p.loglex())

    def met_au_net(self):
        """Remove identical patterns and repetitions in patterns."""
        self.__lex.met_au_net()

    def __delitem__(self, i):
        "Remove the descriptor number i ( x.__delitem__(i) <==> del x[i] )."
        if i in self.ls_num():
            self.__lex.enleve(i)

###################################
#### calculs


    def prediction(self, D, **kw):
        """Return the prediction of the best descriptor on data D,

Keyword arguments:
deb -- first position of the computed segment of the data (default:0);
fin -- last position of the computed segment of the date
    (default: the last one).
"""
        
        d=kw.get("deb",0)
        f=kw.get("fin",-2)
        if isinstance(D, matrice._Matr):
            return self.__lex.pred(D._Matr__c_elem(),d,f)
        elif isinstance(D, sequence._Seq):
            return self.__lex.pred(D._Seq__c_elem(),d,f)

    def ls_evalue(self, D, **kw):
        """Return the dictionnary {tuple of descriptors
numbers:prediction of this (pattern of) descriptor on D}.

Keyword arguments:
deb -- first position of the computed segment of the data (default:0);
fin -- last position of the computed segment of the date
       (default: the last one).
"""
        d=kw.get("deb",0)
        f=kw.get("fin",-2)
        
        if isinstance(D, matrice._Matr):
            rep=self.__lex.ecrit_evalue(D._Matr__c_elem(),d,f)
        elif isinstance(D, sequence._Seq):
            rep=self.__lex.ecrit_evalue(D._Seq__c_elem(),d,f)
        if rep!="":
            return eval(rep)    
        return {}
        
    def windows(self, D, t, p):
        """On data D, return the list of best predictions on a sliding
window of size t with slidings steps of size p.
"""
        lv=[]
        lg=len(D)-t+1
        i=0
        while i<lg:
            lv.append(self.prediction(D,deb=i,fin=i+t-1))
            i+=p
        return lv

    def __c_elem(self):
        return self.__lex;

############################################
###   sorties
            
    def __str__(self):
        """(x.__str__()<==> str(x))"""
        return str(self.__lex)
        
    def __len__(self):
        "Return the number of descriptors."
        return self.__lex.nb_desc()

    def ls_num(self):
        "Return the list of descriptors numbers."
        return eval(self.__lex.ls_num())

    def is_empty(self):
        return self.__lex.est_vide()==1

    def __iter__(self):
        "Iterator on descriptors. Use:  for element in l."
        return Lexique_iter(self)
            
    def __getitem__(self, i):
        """Return the string of descriptor number i, empty string if
it does not exist (x.__getitem__(i) <==> x[i]).
"""
        return self.__lex.__getitem__(i)

    


#################################################
############################

class Lexique_iter:

    def __init__(self,lx):
        self.__lex=lx
        self.__lnum=self.__lex.ls_num()
        self.__ln=len(self.__lnum)
        self.__ind=-1

    def next(self):
        if self.__ind==self.__ln-1:
            raise StopIteration
        else:
            self.__ind+=1
            return self.__lex[self.__lnum[self.__ind]]
        
            
